//
//  BlueToothViewController.m
//  demo
//
//  Created by gg.lucy on 2018/10/9.
//  Copyright © 2018 王刚. All rights reserved.
//
//实现iOS蓝牙中心设备
#import "BlueToothViewController.h"
#import <CoreBluetooth/CoreBluetooth.h>
#define SERVICE_UUID        @"CDD1"
#define CHARACTERISTIC_UUID @"CDD2"

#import "GGTextField.h"
#import "GGFont.h"
#import <Masonry.h>
@interface BlueToothViewController ()<CBCentralManagerDelegate,CBPeripheralDelegate>
@property(nonatomic,strong)CBCentralManager * centralManager;
@property(nonatomic,strong)CBPeripheral * peripheral;
@property(nonatomic,strong)CBCharacteristic * characteristic;


@property(nonatomic,strong)GGTextField * centerTF;
@end

@implementation BlueToothViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor=[UIColor cyanColor];
    //创建中心设备管理,会回调centralManagerDidUpdateState
    self.centralManager=[[CBCentralManager alloc]initWithDelegate:self queue:dispatch_get_main_queue()];
    
    
   self.centerTF= [[GGTextField alloc]initWithFrame:CGRectMake(10, navigationBarHeight+10, kScreenWidth-20, 50) font:13 keyboardType:UIKeyboardTypeDefault placeholder:@"请输入文字" GGTextFieldType:4 cornwith:5];
    [self.view addSubview:self.centerTF];
    UIButton * postBtn =[UIButton buttonWithType:UIButtonTypeCustom];
    [postBtn setTitle:@"post" forState:UIControlStateNormal];
    [postBtn setTitleColor:[UIColor redColor] forState:UIControlStateNormal];
    postBtn.titleLabel.font=[GGFont ggFontWithSize:15];
    postBtn.titleLabel.textAlignment=NSTextAlignmentCenter;
    [postBtn addTarget:self action:@selector(postBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:postBtn];
    [postBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.centerTF.mas_left).with.offset(10);
        make.top.equalTo(self.centerTF.mas_bottom).with.offset(10);
        make.size.mas_offset(CGSizeMake(100, 30));
    }];
    UIButton * getBtn =[UIButton buttonWithType:UIButtonTypeCustom];
    [getBtn setTitle:@"get" forState:UIControlStateNormal];
    [getBtn setTitleColor:[UIColor redColor] forState:UIControlStateNormal];
    getBtn.titleLabel.font=[GGFont ggFontWithSize:15];
    getBtn.titleLabel.textAlignment=NSTextAlignmentCenter;
    [getBtn addTarget:self action:@selector(getBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:getBtn];
    [getBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.view.mas_right).with.offset(-10);
        make.top.equalTo(self.centerTF.mas_bottom).with.offset(10);
        make.size.mas_offset(CGSizeMake(100, 30));
    }];
    // Do any additional setup after loading the view.
}
-(void)centralManagerDidUpdateState:(CBCentralManager *)central{
    //蓝牙可用 开始扫描外设
    if (central.state == CBManagerStatePoweredOn) {
        NSLog(@"蓝牙可用");
        //根据SERVICE_UUID来扫描外设,如果不设置SERVICE_UUID,则扫描所有蓝牙设备
        [central scanForPeripheralsWithServices:@[[CBUUID UUIDWithString:SERVICE_UUID]] options:nil];
    }else if (central.state == CBCentralManagerStateUnsupported){
        NSLog(@"该设备不支持蓝牙");
    }else if (central.state == CBCentralManagerStatePoweredOff){
        NSLog(@"该设备蓝牙");
    }
}
//5当扫描到外设之后,就会回调下面的这个方法,可以在这个方法中继续设置筛选条件,例如根据外设名字的前缀来选择,如果符合条件就进行连接
//发现符合要求的外设，就会回调
-(void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(nonnull CBPeripheral *)peripheral advertisementData:(nonnull NSDictionary<NSString *,id> *)advertisementData RSSI:(nonnull NSNumber *)RSSI{
    //对外设对象进行强引用
    self.peripheral=peripheral;
    if ([peripheral.name hasPrefix:@"WH"]) {
        //可以根据外设的名字来过滤外设
        [central connectPeripheral:peripheral options:nil];
    }
    //连接外设
    [central connectPeripheral:peripheral options:nil];
}
//7当连接成功时候,就会来到下面这个方法,为了省电,当连接上外设之后,就让中心设备停止扫描,并且别忘记设置连接上的外设的代理,在这个方法里根据UUID进行服务来查找
//连接成功
-(void)centralManager:(CBCentralManager *)central  didConnectPeripheral:(nonnull CBPeripheral *)peripheral{
    //可以停止扫描
    [self.centralManager stopScan];
    //设置代理
    peripheral.delegate=self;
    //根据UUID来寻找服务‘
    [peripheral discoverServices:@[[CBUUID UUIDWithString:SERVICE_UUID]]];
    NSLog(@"连接成功");
    
}
//8连接失败和断开连接也有各自的回调的方法 在断开连接的时候 我么可以设置自动重连 根据项目需求来自定义里面的代码
-(void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error{
    NSLog(@"连接失败");
}
//断开连接
-(void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error{
    NSLog(@"断开连接");
    //断开连接可以设置重新连接
    [central connectPeripheral:peripheral options:nil];
}
//9下面开始处理方法
/*最开始就是发现服务的方法这个方法可以遍历服务,找到需要的服务,由于上面做的外设只有一个服务,所以我这里直接取服务中最后一个lastObject就行了 找到服务之后连贯的动作继续根据特征的UUID寻找服务中的特征
  */
//发现服务
-(void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error{
    //遍历出外设中所有的服务
    for (CBService * service  in peripheral.services) {
        NSLog(@"所有的服务:%@",service);
    }
    //这里只有一个服务，所有直接获取
    CBService *service =peripheral.services.lastObject;
    //根据UUID获取服务的特征
    [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID]] forService:service];
}
//10下面的这个方法做的事情不少
/*当发现特征之后,与服务一样可以遍历特征,根据外设开发人员给的文档找出不同特征，做出相应的操作'
 我的外设只设置一个特征,所以也是直接通过lastOBject拿到特征
 重要:一般开发中可以设置两个特征,一个用来发送数据,一个用来接收中心设备设备写过啦的数据
 这里用一个属性引用特征,是为了通过这个特征向外设写入数据活着发送指令
 readValueForCharacteristic方法是直接读一次这个特征上的数据
 */
//发送特征的回调
-(void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(nonnull CBService *)service error:(nullable NSError *)error{
    //遍历所需要的特征
    for (CBCharacteristic * characterstic in service.characteristics) {
        NSLog(@"所有的特征:%@",characterstic);
        //从外设开发人员那里拿到不同的UUID不同的特征做不同的事情,比如有读取数据的特征,也有写入数据的特征
    }
    //从这里只获取一个特征,写入数据的时候需要用到这个特征
    self.characteristic=service.characteristics.lastObject;
    //直接读取这个特征数据,会调用didUpdateValueForCharacteristic
    [peripheral readValueForCharacteristic:self.characteristic];
    //订阅通知
    /*setNotifyValue:(BOOL)enabled forCharacteristic:(CBCharacteristic *)characteristic方法
      是对这个特征进行订阅，订阅成功之后,就可以监控外设中这个特征值得变化了
     
     */
    [peripheral setNotifyValue:YES forCharacteristic:self.characteristic];
}
//11当订阅状态的改变
-(void)peripheral:(CBPeripheral *)peripheral  didUpdateNotificationStateForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error{
    if (error) {
        NSLog(@"订阅失败");
        NSLog(@"%@",error);
    }
    if (characteristic.isNotifying) {
        NSLog(@"订阅成功");
    }else{
        NSLog(@"取消订阅");
    }
}
//12外设可以发送数据中心设备  中心设备也可以从外设读取数据,当发生这些事情的时候,就会回调这个方法,通过特征中的value属性拿到原始数据,然后根据需求解析数据
/*接收到数据回调*/
-(void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error{
    //拿到外设发送过去的数据
    NSData * data =characteristic.value;
    self.centerTF.text=[[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding];
}
//13中心设备可以向外设写入数据,也可以向外设发送请求或指令,当需要进行这些操作的时候该怎么办
/*
 1.首先把写入的数据转化为NSData格式,然后根据上面拿到的写入数据的特征,运用方法writeValue:(nsdata*)data forCharacteristic:(CBCharacteristic *)characteristic type:(CBCharacteristicWriteType)type来进行数据的写入
 2.当写入数据的时候，系统会回调这个方法peripheral:(CBPeripheral *)peripheral didWriteValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error;
 */
//写入数据
-(void)postBtnClick:(UIButton *)post{
    //用NSdata类型来写入
    NSData * data =[self.centerTF.text dataUsingEncoding:NSUTF8StringEncoding];
    //根据上面的特征self.characteristic来写入数据
    [self.peripheral writeValue:data forCharacteristic:self.characteristic type:CBCharacteristicWriteWithResponse];
}
//14中心设备如何主动从外设读取数据
/*
 1.用正在连接的外设对象来调用readValueForCharacteristic方法 并且把将要读取数据的特征作为参数，这样就可以主动那一次数据
   去到第12步回调方法中在特征额value属性中拿到这次的数据
 */
//读取数据
-(void)getBtnClick:(UIButton *)get{
    [self.peripheral readValueForCharacteristic:self.characteristic];
}
- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}
@end
